#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class VeteranDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "veterans"
    UNIVERSE = "civilian veterans 18 years and over"
    RAW_TABLE_NAME = 'B21002'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001": "universe"
    })



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
import census_data_aggregator
from census_data_downloader.core import MOE_MAP
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class ForeignBornDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "foreignborn"
    UNIVERSE = "total population"
    RAW_TABLE_NAME = 'B05002'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001": "universe",
        "002": "native",
        "013": "foreign_born"
    })


@register
class CitizenDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "citizenstatus"
    UNIVERSE = "total population"
    RAW_TABLE_NAME = "B05001"
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001": "universe",
        "002": "us_citizen_born_us",
        "003": "us_citizen_born_puertorico_or_us_island",
        "004": "us_citizen_born_abroad_american_parents",
        "005": "us_citizen_by_naturalization",
        "006": "not_us_citizen"
    })

    def process(self, df):
        def calculate_moe(row):
            # our custom groups
            if row['us_citizen_by_naturalization_moe'] in list(MOE_MAP.values()):
                value = sum([row['us_citizen_by_naturalization'], row['us_citizen_born_abroad_american_parents'], row['us_citizen_born_puertorico_or_us_island_alone'], row['us_citizen_born_us']])
                moe = None
            value, moe = census_data_aggregator.approximate_sum(
                (row['us_citizen_by_naturalization'], row['us_citizen_by_naturalization_moe']),
                (row['us_citizen_born_abroad_american_parents'], row['us_citizen_born_abroad_american_parents_moe']),
                (row['us_citizen_born_puertorico_or_us_island_alone'], row['us_citizen_born_puertorico_or_us_island_moe']),
                (row['us_citizen_born_us'], row['us_citizen_born_us_moe']),
            )
            row['us_citizen_total'] = value
            row['us_citizen_total_moe'] = moe
            return row

        df = df.apply(
            calculate_moe,
            axis=1
        )

        # Pass it back
        return df



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Command-line interface.
"""
import click
from census_data_downloader.tables import TABLE_LIST
TABLES_LOOKUP = dict((k.PROCESSED_TABLE_NAME, k) for k in TABLE_LIST)


@click.group(help="Download Census data and reformat it for humans")
@click.argument(
    "table",
    nargs=1,
    required=True
)
@click.option(
    "--data-dir",
    nargs=1,
    default="./",
    help="The folder where you want to download the data"
)
@click.option(
    "--year",
    default=None,
    type=int,
    help="The years of data to download. By default it gets only the latest year. Not all data are available for every year. Submit 'all' to get every year."
)
@click.option(
    '--force',
    is_flag=True,
    help="Force the downloading of the data"
)
@click.pass_context
def cmd(ctx, table, data_dir="./", year=None, force=False):
    ctx.ensure_object(dict)
    ctx.obj['table'] = table
    ctx.obj['data_dir'] = data_dir
    ctx.obj['year'] = year
    ctx.obj['force'] = force
    try:
        klass = TABLES_LOOKUP[ctx.obj['table']]
    except KeyError:
        raise click.ClickException("Table not found")
    ctx.obj['klass'] = klass
    ctx.obj['runner'] = klass(
        data_dir=data_dir,
        years=year,
        force=force
    )


@cmd.command(help="Download nationwide data")
@click.pass_context
def nationwide(ctx):
    ctx.obj['runner'].download_nationwide()


@cmd.command(help="Download divisions")
@click.pass_context
def divisions(ctx):
    ctx.obj['runner'].download_divisions()


@cmd.command(help="Download regions")
@click.pass_context
def regions(ctx):
    ctx.obj['runner'].download_regions()


@cmd.command(help="Download states")
@click.pass_context
def states(ctx):
    ctx.obj['runner'].download_states()


@cmd.command(help="Download Congressional districts")
@click.pass_context
def congressionaldistricts(ctx):
    ctx.obj['runner'].download_congressional_districts()


@cmd.command(help="Download statehouse districts")
@click.pass_context
def statelegislativedistricts(ctx):
    ctx.obj['runner'].download_state_legislative_upper_districts()
    ctx.obj['runner'].download_state_legislative_lower_districts()


@cmd.command(help="Download counties in all states")
@click.pass_context
def counties(ctx):
    ctx.obj['runner'].download_counties()


@cmd.command(help="Download Census-designated places")
@click.pass_context
def places(ctx):
    ctx.obj['runner'].download_places()


@cmd.command(help="Download urban areas")
@click.pass_context
def urbanareas(ctx):
    ctx.obj['runner'].download_urban_areas()


@cmd.command(help="Download metropolitan statistical areas")
@click.pass_context
def msas(ctx):
    ctx.obj['runner'].download_msas()


@cmd.command(help="Download combined statistical areas")
@click.pass_context
def csas(ctx):
    ctx.obj['runner'].download_csas()


@cmd.command(help="Download public use microdata areas")
@click.pass_context
def pumas(ctx):
    ctx.obj['runner'].download_pumas()


@cmd.command(help="Download New England city and town areas")
@click.pass_context
def nectas(ctx):
    ctx.obj['runner'].download_nectas()


@cmd.command(help="Download combined New England city and town areas")
@click.pass_context
def cnectas(ctx):
    ctx.obj['runner'].download_cnectas()


@cmd.command(help="Download American Indian, Alaska Native and Native Hawaiian homelands")
@click.pass_context
def aiannhhomelands(ctx):
    ctx.obj['runner'].download_aiannh_homelands()


@cmd.command(help="Download Census tracts")
@click.pass_context
def tracts(ctx):
    ctx.obj['runner'].download_tracts()


@cmd.command(help="Download ZIP Code tabulation areas")
@click.pass_context
def zctas(ctx):
    ctx.obj['runner'].download_zctas()


@cmd.command(help="Download unified school districts")
@click.pass_context
def unifiedschooldistricts(ctx):
    ctx.obj['runner'].download_unified_school_districts()


@cmd.command(help="Download elementary school districts")
@click.pass_context
def elementaryschooldistricts(ctx):
    ctx.obj['runner'].download_elementary_school_districts()


@cmd.command(help="Download secondary school districts")
@click.pass_context
def secondaryschooldistricts(ctx):
    ctx.obj['runner'].download_secondary_school_districts()


@cmd.command(help="Download Alaska Native regional corporations")
@click.pass_context
def alaskanative(ctx):
    ctx.obj['runner'].download_alaska_native()


@cmd.command(help="Download county subdivisions")
@click.pass_context
def countysubdivision(ctx):
    ctx.obj['runner'].download_county_subdivision()


@cmd.command(help="Download everything from everywhere")
@click.pass_context
def everything(ctx):
    ctx.obj['runner'].download_everything()


if __name__ == '__main__':
    cmd()



from census_data_downloader.core.decorators import TABLE_LIST
from .age import AgeDownloader
from .ancestry import AncestryDownloader
from .classofworker import ClassOfWorkerDownloader
from .disability import (
    DisabilityDownloader,
    DisabilityPovertyDownloader,
    CognitiveDifficultyDownloader
)
from .education import (
    EducationDownloader,
    EducationShortDownloader
)
from .employmentstatus import EmploymentStatusDownloader
from .foreignborn import (
    ForeignBornDownloader,
    CitizenDownloader
)
from .gini import GiniIndexDownloader
from .housing import (
    HousingValueDownloader,
    MedianMonthlyHousingCostsDownloader,
    MedianHousingValueDownloader,
    MedianGrossRentDownloader,
    TenureDownloader,
    TenurePopDownloader,
    TenureLatinoDownloader,
    TenureWhiteDownloader,
    TenureBlackDownloader,
    TenureAsianDownloader,
    MedianRentBedroomDownloader,
    GrossRentBedroomDownloader
)
from .householdincome import (
    HouseholdIncomeDownloader,
    HouseholdIncomeLatinoDownloader,
    HouseholdIncomeWhiteDownloader,
    HouseholdIncomeBlackDownloader,
    HouseholdIncomeAsianDownloader
)
from .internet import InternetDownloader
from .language import (
    HouseholdLanguageDownloader,
    LanguageShortFormDownloader,
    LanguageLongFormDownloader
)
from .latino import LatinoDownloader
from .medianage import MedianAgeDownloader
from .medianhouseholdincome import (
    MedianHouseholdIncomeDownloader,
    MedianHouseholdIncomeLatinoDownloader,
    MedianHouseholdIncomeWhiteDownloader,
    MedianHouseholdIncomeBlackDownloader,
    MedianHouseholdIncomeAsianDownloader
)
from .mobility import (
    MobilityDownloader,
    MobilityBySexDownloader,
    MobilityWhiteDownloader,
    MobilityBlackDownloader,
    MobilityAsianDownloader,
    MobilityLatinoDownloader,
    MobilityByCitizenship
)
from .percapitaincome import (
    PerCapitaIncomeDownloader,
    PerCapitaIncomeLatinoDownloader,
    PerCapitaIncomeWhiteDownloader,
    PerCapitaIncomeBlackDownloader,
    PerCapitaIncomeAsianDownloader
)
from .population import PopulationDownloader
from .poverty import (
    PovertyDownloader,
    PovertyBySexDownloader,
    PovertyAgeDownloader,
    PovertyLatinoDownloader,
    PovertyWhiteDownloader,
    PovertyBlackDownloader,
    PovertyAsianDownloader
)
from .race import (
    RaceDownloader,
    AmericanIndianAlaskaNativeAloneOrInComboDownloader,
    AsianDownloader
)
from .snap import (
    SnapbyPovertyDownloader,
    SnapLatinoDownloader,
    SnapWhiteDownloader,
    SnapBlackDownloader,
    SnapAsianDownloader,
)
from .veteran import VeteranDownloader
from .yearstructurebuilt import (
    YearStructureBuiltDownloader,
    TenureByYearStructureBuiltDownloader,
    AgeHouseholderByYearBuiltDownloader
)
from .votingage import VoterDownloader

__all__ = [
    TABLE_LIST,
    AgeDownloader,
    AgeHouseholderByYearBuiltDownloader,
    AncestryDownloader,
    AsianDownloader,
    CitizenDownloader,
    ClassOfWorkerDownloader,
    CognitiveDifficultyDownloader,
    DisabilityDownloader,
    DisabilityPovertyDownloader,
    EducationDownloader,
    EducationShortDownloader,
    EmploymentStatusDownloader,
    ForeignBornDownloader,
    GiniIndexDownloader,
    GrossRentBedroomDownloader,
    HousingValueDownloader,
    HouseholdIncomeDownloader,
    HouseholdIncomeLatinoDownloader,
    HouseholdIncomeWhiteDownloader,
    HouseholdIncomeBlackDownloader,
    HouseholdIncomeAsianDownloader,
    InternetDownloader,
    HouseholdLanguageDownloader,
    LanguageShortFormDownloader,
    LanguageLongFormDownloader,
    LatinoDownloader,
    MedianAgeDownloader,
    MedianGrossRentDownloader,
    MedianHouseholdIncomeDownloader,
    MedianHouseholdIncomeLatinoDownloader,
    MedianHouseholdIncomeWhiteDownloader,
    MedianHouseholdIncomeBlackDownloader,
    MedianHouseholdIncomeAsianDownloader,
    MedianHousingValueDownloader,
    MedianMonthlyHousingCostsDownloader,
    MedianRentBedroomDownloader,
    MobilityDownloader,
    MobilityBySexDownloader,
    MobilityWhiteDownloader,
    MobilityBlackDownloader,
    MobilityAsianDownloader,
    MobilityLatinoDownloader,
    MobilityByCitizenship,
    PerCapitaIncomeDownloader,
    PerCapitaIncomeLatinoDownloader,
    PerCapitaIncomeWhiteDownloader,
    PerCapitaIncomeBlackDownloader,
    PerCapitaIncomeAsianDownloader,
    PopulationDownloader,
    PovertyDownloader,
    PovertyBySexDownloader,
    PovertyAgeDownloader,
    PovertyLatinoDownloader,
    PovertyWhiteDownloader,
    PovertyBlackDownloader,
    PovertyAsianDownloader,
    RaceDownloader,
    SnapbyPovertyDownloader,
    SnapLatinoDownloader,
    SnapWhiteDownloader,
    SnapBlackDownloader,
    SnapAsianDownloader,
    AmericanIndianAlaskaNativeAloneOrInComboDownloader,
    TenureDownloader,
    TenurePopDownloader,
    TenureLatinoDownloader,
    TenureWhiteDownloader,
    TenureBlackDownloader,
    TenureAsianDownloader,
    TenureByYearStructureBuiltDownloader,
    VeteranDownloader,
    VoterDownloader,
    YearStructureBuiltDownloader
]



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from pprint import pprint
import census_data_aggregator
from census_data_downloader.core import MOE_MAP
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class RaceDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "race"
    UNIVERSE = "total population"
    RAW_TABLE_NAME = 'B03002'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '003': "white_alone",
        '004': "black_alone",
        '005': "american_indian_and_alaska_native",
        '006': "asian_alone",
        '007': "native_hawaiian_and_pacific_islander",
        '008': "other_alone",
        '009': "two_or_more_races",
        '012': "latino_alone"
    })

    def process(self, df):
        # Our custom race groups
        def calculate_asians_moe(row):
            pprint(row)
            if row['asian_alone_moe'] in list(MOE_MAP.values()):
                value = sum([row['asian_alone'], row['native_hawaiian_and_pacific_islander']])
                moe = None
            value, moe = census_data_aggregator.approximate_sum(
                (row['native_hawaiian_and_pacific_islander'], row['native_hawaiian_and_pacific_islander_moe']),
                (row['asian_alone'], row['asian_alone_moe']),
            )
            row['asians_all'] = value
            row['asians_all_moe'] = moe
            return row

        df = df.apply(
            calculate_asians_moe,
            axis=1
        )

        def calculate_other_moe(row):
            pprint(row)
            if row['american_indian_and_alaska_native_moe'] in list(MOE_MAP.values()):
                value = sum([row['american_indian_and_alaska_native'], row['other_alone'], row['two_or_more_races']])
                moe = None
            value, moe = census_data_aggregator.approximate_sum(
                (row['american_indian_and_alaska_native'], row['american_indian_and_alaska_native_moe']),
                (row['other_alone'], row['other_alone_moe']),
                (row['two_or_more_races'], row['two_or_more_races_moe']),
            )
            row['other_all'] = value
            row['other_all_moe'] = moe
            return row

        df = df.apply(
            calculate_other_moe,
            axis=1
        )

        # Pass it back
        return df


@register
class AmericanIndianAlaskaNativeAloneOrInComboDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = 'aianaloneorincombo'
    UNIVERSE = "people who are american indian or alaska native alone or in combination with one or more races"
    RAW_TABLE_NAME = 'B02010'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001": "universe"
    })


@register
class AsianDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "asian"
    UNIVERSE = "total asian alone population"
    RAW_TABLE_NAME = 'B02015'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '002': "asian_indian",
        '003': "bangladeshi",
        '004': "bhutanese",
        '005': "burmese",
        '006': "cambodian",
        '007': "chinese_except_taiwanese",
        '008': "filipino",
        '009': "hmong",
        '010': "indonesian",
        '011': "japanese",
        '012': "korean",
        '013': "laotian",
        '014': "malaysian",
        '015': "mongolian",
        '016': "nepalese",
        '017': "okinawan",
        '018': "pakistani",
        '019': "sri_lankan",
        '020': "taiwanese",
        '021': "thai",
        '022': "vietnamese",
        '023': "other_asian_specified",
        '024': "other_asian_not_specified",
        '025': "two_or_more_asian",
    })



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class PerCapitaIncomeDownloader(BaseTableConfig):
    # inflation adjusted
    PROCESSED_TABLE_NAME = "percapitaincome"
    UNIVERSE = "total population"
    RAW_TABLE_NAME = 'B19301'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "per_capita_income"
    })


@register
class PerCapitaIncomeLatinoDownloader(PerCapitaIncomeDownloader):
    # inflation adjusted
    PROCESSED_TABLE_NAME = "percapitaincomelatino"
    RAW_TABLE_NAME = 'B19301I'


@register
class PerCapitaIncomeWhiteDownloader(PerCapitaIncomeDownloader):
    # inflation adjusted
    PROCESSED_TABLE_NAME = "percapitaincomewhite"
    RAW_TABLE_NAME = 'B19301H'


@register
class PerCapitaIncomeBlackDownloader(PerCapitaIncomeDownloader):
    # inflation adjusted
    PROCESSED_TABLE_NAME = "percapitaincomeblack"
    RAW_TABLE_NAME = 'B19301B'


@register
class PerCapitaIncomeAsianDownloader(PerCapitaIncomeDownloader):
    # inflation adjusted
    PROCESSED_TABLE_NAME = "percapitaincomeasian"
    RAW_TABLE_NAME = 'B19301D'



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class YearStructureBuiltDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "yearstructurebuilt"
    UNIVERSE = "housing units"
    RAW_TABLE_NAME = 'B25034'

    @property
    def RAW_FIELD_CROSSWALK(self):
        if self._year > 2014:
            return collections.OrderedDict({
                '001': 'universe',
                '002': 'built_2014_or_later',
                '003': 'built_2010_to_2013',
                '004': 'built_2000_to_2009',
                '005': 'built_1990_to_1999',
                '006': 'built_1980_to_1989',
                '007': 'built_1970_to_1979',
                '008': 'built_1960_to_1969',
                '009': 'built_1950_to_1959',
                '010': 'built_1940_to_1949',
                '011': 'built_1929_or_earlier'
            })
        elif self._year > 2011:
            return collections.OrderedDict({
                '001': 'universe',
                '002': 'built_2010_or_later',
                '003': 'built_2000_to_2009',
                '004': 'built_1990_to_1999',
                '005': 'built_1980_to_1989',
                '006': 'built_1970_to_1979',
                '007': 'built_1960_to_1969',
                '008': 'built_1950_to_1959',
                '009': 'built_1940_to_1949',
                '010': 'built_1939_or_earlier'
            })
        else:
            return collections.OrderedDict({
                '001': 'universe',
                '002': 'built_2005_or_later',
                '003': 'built_2000_to_2004',
                '004': 'built_1990_to_1999',
                '005': 'built_1980_to_1989',
                '006': 'built_1970_to_1979',
                '007': 'built_1960_to_1969',
                '008': 'built_1950_to_1959',
                '009': 'built_1940_to_1949',
                '010': 'built_1939_or_earlier'
            })


@register
class AgeHouseholderByYearBuiltDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "agehouseholderbyyearbuilt"
    UNIVERSE = "occupied housing units"
    RAW_TABLE_NAME = 'B25126'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': 'estimate',
        '002': 'total_owner',
        '003': 'owner_15to34yrsold',
        '004': 'owner_15to34yrsoldbuilt2014orlater',
        '005': 'owner_15to34yrsoldbuilt2010to2013',
        '006': 'owner_15to34yrsoldbuilt2000to2009',
        '007': 'owner_15to34yrsoldbuilt1990to1999',
        '008': 'owner_15to34yrsoldbuilt1980to1989',
        '009': 'owner_15to34yrsoldbuilt1970to1979',
        '010': 'owner_15to34yrsoldbuilt1960to1969',
        '011': 'owner_15to34yrsoldbuilt1950to1959',
        '012': 'owner_15to34yrsoldbuilt1940to1949',
        '013': 'owner_15to34yrsoldbuilt1939orearlier',
        '014': 'owner_35to64yrsold',
        '015': 'owner_35to64yrsoldbuilt2014orlater',
        '016': 'owner_35to64yrsoldbuilt2010to2013',
        '017': 'owner_35to64yrsoldbuilt2000to2009',
        '018': 'owner_35to64yrsoldbuilt1990to1999',
        '019': 'owner_35to64yrsoldbuilt1980to1989',
        '020': 'owner_35to64yrsoldbuilt1970to1979',
        '021': 'owner_35to64yrsoldbuilt1960to1969',
        '022': 'owner_35to64yrsoldbuilt1950to1959',
        '023': 'owner_35to64yrsoldbuilt1940to1949',
        '024': 'owner_35to64yrsoldbuilt1939orearlier',
        '025': 'owner_65older',
        '026': 'owner_65older2014orlater',
        '027': 'owner_65olderbuilt2010to2013',
        '028': 'owner_65olderbuilt2000to2009',
        '029': 'owner_65olderbuilt1990to1999',
        '030': 'owner_65olderbuilt1980to1989',
        '031': 'owner_65olderbuilt1970to1979',
        '032': 'owner_65olderbuilt1960to1969',
        '033': 'owner_65olderbuilt1950to1959',
        '034': 'owner_65olderbuilt1940to1949',
        '035': 'owner_65olderbuilt1939orearlier',
        '036': 'total_renter',
        '037': 'renter_15to34yrsold',
        '038': 'renter_15to34yrsoldbuilt2014orlater',
        '039': 'renter_15to34yrsoldbuilt2010to2013',
        '040': 'renter_15to34yrsoldbuilt2000to2009',
        '041': 'renter_15to34yrsoldbuilt1990to1999',
        '042': 'renter_15to34yrsoldbuilt1980to1989',
        '043': 'renter_15to34yrsoldbuilt1970to1979',
        '044': 'renter_15to34yrsoldbuilt1960to1969',
        '045': 'renter_15to34yrsoldbuilt1950to1959',
        '046': 'renter_15to34yrsoldbuilt1940to1949',
        '047': 'renter_15to34yrsoldbuilt1939orearlier',
        '048': 'renter_35to64yrsold',
        '049': 'renter_35to64yrsold2014orlater',
        '050': 'renter_35to64yrsold2010to2013',
        '051': 'renter_35to64yrsold2000to2009',
        '052': 'renter_35to64yrsold1990to1999',
        '053': 'renter_35to64yrsold1980to1989',
        '054': 'renter_35to64yrsold1970to1979',
        '055': 'renter_35to64yrsoldbuilt1960to1969',
        '056': 'renter_35to64yrsoldbuilt1950to1959',
        '057': 'renter_35to64yrsoldbuilt1940to1949',
        '058': 'renter_35to64yrsoldbuilt1939orearlier',
        '059': 'renter_65older',
        '060': 'renter_65olderbuilt2014orlater',
        '061': 'renter_65olderbuilt2010to2013',
        '062': 'renter_65olderbuilt2000to2009',
        '063': 'renter_65olderbuilt1990to1999',
        '064': 'renter_65olderbuilt1980to1989',
        '065': 'renter_65olderbuilt1970to1979',
        '066': 'renter_65olderbuilt1960to1969',
        '067': 'renter_65olderbuilt1950to1959',
        '068': 'renter_65olderbuilt1940to1949',
        '069': 'renter_65olderbuilt1939orearlier'
    })


@register
class TenureByYearStructureBuiltDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "tenurebyyearbuilt"
    UNIVERSE = "occupied housing units"
    RAW_TABLE_NAME = 'B25036'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': 'estimate',
        '002': 'owner',
        '003': 'owner_built2014orlater',
        '004': 'owner_built2010to2013',
        '005': 'owner_built2000to2009',
        '006': 'owner_built1990to1999',
        '007': 'owner_built1980to1989',
        '008': 'owner_built1970to1979',
        '009': 'owner_built1960to1969',
        '010': 'owner_built1950to1959',
        '011': 'owner_built1940to1949',
        '012': 'owner_built1939orearlier',
        '013': 'renter',
        '014': 'renter_built2014orlater',
        '015': 'renter_built2010to2013',
        '016': 'renter_built2000to2009',
        '017': 'renter_built1990to1999',
        '018': 'renter_built1980to1989',
        '019': 'renter_built1970to1979',
        '020': 'renter_built1960to1969',
        '021': 'renter_built1950to1959',
        '022': 'renter_built1940to1949',
        '023': 'renter_built1939orearlier'
    })



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class LatinoDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "latino"
    UNIVERSE = "total population"
    RAW_TABLE_NAME = 'B03001'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '002': "not_hispanic_or_latino",
        '003': "hispanic_or_latino",
        '004': "mexican",
        '005': "puerto_rican",
        '006': "cuban",
        '007': "dominican",
        '008': "central_american",
        '009': "costa_rican",
        '010': "guatemalan",
        '011': "honduran",
        '012': "nicaraguan",
        '013': "panamanian",
        '014': "salvadoran",
        '015': "other_central_american",
        '016': "south_american",
        '017': "argentinean",
        '018': "bolivian",
        '019': "chilean",
        '020': "colombian",
        '021': "ecuadorian",
        '022': "paraguayan",
        '023': "peruvian",
        '024': "uruguayan",
        '025': "venezuelan",
        '026': "other_south_american",
        '027': "other_hispanic_or_latino",
        '028': "spaniard",
        '029': "spanish",
        '030': "spanish_american",
        '031': "all_other_hispanic_or_latino"
    })



#! /usr/bin/env python
# -*- coding: utf-8 -*
"""
Decorators to help manage our custom classes.
"""
TABLE_LIST = []


def register(cls):
    """
    A decorator to register new table configuration classes.
    """
    TABLE_LIST.append(cls)
    return cls


def downloader(func):
    """
    A decorator to download data inside a table configuration class.
    """
    def inner(*args, **kwargs):
        # Grab the TableConfig
        table_config = args[0]
        # Grab the geotype downloader class by running the metaprogramming function
        downloader_klass = func(table_config)
        # For each year authorized on the config
        for year in table_config.years_to_download:
            # Create the geotype downloader instance
            downloader = downloader_klass(table_config, year)
            # Download the raw data
            downloader.download()
            # Process the data
            downloader.process()
    return inner
